@@ -43,6 +43,12 @@ domReadWellBalancedString( xmlDocPtr doc, xmlChar* string );
 
 /* A.1 DOM specified section */
 
+const xmlChar *
+domName( xmlNodePtr node );
+
+void
+domSetName( xmlNodePtr node, xmlChar* name );
+
 xmlNodePtr
 domAppendChild( xmlNodePtr self,
                 xmlNodePtr newChild );
@@ -104,7 +110,15 @@ domImportNode( xmlDocPtr document, xmlNodePtr node, int move );
 xmlNodeSetPtr
 domGetElementsByTagName( xmlNodePtr self, xmlChar* name );
 
+xmlNodeSetPtr
+domGetElementsByTagNameNS( xmlNodePtr self, xmlChar* nsURI, xmlChar* name );
+
 xmlNodePtr
 domSetOwnerDocument( xmlNodePtr self, xmlDocPtr doc );
 
+xmlNsPtr
+domNewNs ( xmlNodePtr elem , xmlChar *prefix, xmlChar *href );
+
+xmlAttrPtr
+domHasNsProp(xmlNodePtr node, const xmlChar *name, const xmlChar *namespace);
 #endif
